@@ -25,6 +25,7 @@ else
 directory = "."
 user = "%USER%"
 input = "unix:%AUDISPD_EVENTS%"
+marker = "test-$$"
 
 [auditlog]
 file = "audit.log"
@@ -42,8 +43,8 @@ execve-argv = [ "array" ]
 
 [translate]
 
-universal = false
-user-db = false
+universal = true
+user-db = true
 drop-raw = false
 
 [enrich]
