@@ -18,22 +18,33 @@ use std::time::{Duration, SystemTime};
 
 use anyhow::{anyhow, Context};
 
-use nix::sys::signal::{sigprocmask, SigSet, SigmaskHow::*, Signal::*};
-use nix::unistd::{chown, execve, Uid, User};
+use nix::sys::{
+    signal::{sigprocmask, SigSet, SigmaskHow::*, Signal::*},
+    sysinfo::sysinfo,
+};
+use nix::unistd::{chown, execve, Group, Uid, User};
 #[cfg(target_os = "linux")]
 use nix::unistd::{setresgid, setresuid};
 
 #[cfg(target_os = "linux")]
 use caps::{securebits::set_keepcaps, CapSet, Capability};
 
-use serde::Serialize;
+use serde::{Deserialize, Serialize};
 
-use laurel::coalesce::Coalesce;
+use laurel::coalesce::{self, Coalesce};
 use laurel::config::{Config, Input, Logfile};
+use laurel::json;
 use laurel::logger;
 use laurel::rotate::FileRotate;
 use laurel::types::Event;
 
+const fn build_id() -> &'static str {
+    match option_env!("LAUREL_BUILD_ID") {
+        None => "generic",
+        Some(s) => s,
+    }
+}
+
 #[derive(Default, Serialize)]
 struct Stats {
     lines: u64,
@@ -131,6 +142,14 @@ impl Logger {
                     _ = User::from_name(user)?.ok_or_else(|| anyhow!("user {user} not found"))?;
                     rot = rot.with_user(user);
                 }
+                for group in &def.clone().groups.unwrap_or_default() {
+                    _ = Group::from_name(group)?
+                        .ok_or_else(|| anyhow!("group {group} not found"))?;
+                    rot = rot.with_group(group);
+                }
+                if def.other {
+                    rot = rot.with_other(true);
+                }
                 if let Some(generations) = &def.generations {
                     rot = rot.with_generations(*generations);
                 }
@@ -146,6 +165,98 @@ impl Logger {
     }
 }
 
+#[derive(Default, Serialize, Deserialize)]
+struct AppState<'a> {
+    ts: u64,
+    state: coalesce::State<'a>,
+}
+
+fn read_state(path: &Path, max_age: Duration) -> Option<coalesce::State> {
+    let r = fs::File::open(path)
+        .map_err(|e| {
+            log::error!("Can't open {}: {e}", path.to_string_lossy());
+            e
+        })
+        .ok()?;
+    match json::from_reader::<_, AppState>(r) {
+        Err(e) => {
+            log::error!(
+                "Can't parse state from file {}: {e}",
+                path.to_string_lossy()
+            );
+            None
+        }
+        Ok(s) => {
+            let ts = SystemTime::UNIX_EPOCH + Duration::from_secs(s.ts);
+            let elapsed = ts
+                .elapsed()
+                .map_err(|e| {
+                    log::error!("Can't determine state age: {e}");
+                    e
+                })
+                .ok()?;
+            // If we can't get the uptime, assume 0, i.e.: don't trust state file
+            let uptime = sysinfo().map(|si| si.uptime()).unwrap_or_default();
+            if elapsed > max_age {
+                log::warn!(
+                    "Discarding stale app state {}: elapsed={}s, max-age={}s",
+                    path.to_string_lossy(),
+                    elapsed.as_secs(),
+                    max_age.as_secs(),
+                );
+                None
+            } else if elapsed > uptime {
+                log::error!(
+                    "Discarding stale app state {}: elapsed={}s, uptime={}s",
+                    path.to_string_lossy(),
+                    elapsed.as_secs(),
+                    uptime.as_secs(),
+                );
+                None
+            } else {
+                log::info!(
+                    "Successfully read state file (#inflight={}, #done={} #proc={}, #pid={})",
+                    s.state.inflight.len(),
+                    s.state.done.len(),
+                    s.state.processes.processes.len(),
+                    s.state.processes.current.len(),
+                );
+                Some(s.state)
+            }
+        }
+    }
+}
+
+fn write_state(path: &Path, state: &coalesce::State) {
+    log::info!(
+        "Writing state (#inflight={}, #done={} #proc={}, #pid={})",
+        state.inflight.len(),
+        state.done.len(),
+        state.processes.processes.len(),
+        state.processes.current.len(),
+    );
+    let mut fr = FileRotate::new(path);
+    if let Err(e) = fr
+        .rotate()
+        .and_then(|_| {
+            json::to_writer(
+                &mut fr,
+                &AppState {
+                    ts: SystemTime::UNIX_EPOCH
+                        .elapsed()
+                        .unwrap_or_default()
+                        .as_secs(),
+                    state: state.clone(),
+                },
+            )
+            .map_err(|e| e.into())
+        })
+        .and_then(|_| fr.flush())
+    {
+        log::error!("Error writing state file {}: {e}", path.to_string_lossy());
+    }
+}
+
 fn run_app() -> Result<(), anyhow::Error> {
     let args: Vec<String> = env::args().collect();
 
@@ -162,7 +273,7 @@ fn run_app() -> Result<(), anyhow::Error> {
     }
 
     if matches.opt_present("v") {
-        println!("{}", laurel::VERSION);
+        println!("{} ({})", laurel::VERSION, build_id());
         return Ok(());
     }
 
@@ -211,7 +322,7 @@ fn run_app() -> Result<(), anyhow::Error> {
     };
 
     if matches.opt_present("d") {
-        println!("Laurel {}: Config ok.", laurel::VERSION);
+        println!("Laurel {} ({}): Config ok.", laurel::VERSION, build_id());
         return Ok(());
     }
 
@@ -245,11 +356,8 @@ fn run_app() -> Result<(), anyhow::Error> {
     fs::set_permissions(&dir, PermissionsExt::from_mode(0o755))
         .with_context(|| format!("chmod: {}", dir.to_string_lossy()))?;
 
-    let mut debug_logger = if let Some(l) = &config.debug.log {
-        Some(Logger::new(l, &dir).context("can't create debug logger")?)
-    } else {
-        None
-    };
+    let statefile_path = config.state.file.as_ref().map(|f| dir.join(f));
+
     let mut error_logger = if let Some(def) = &config.debug.parse_error_log {
         let mut filename = dir.clone();
         filename.push(&def.file);
@@ -284,7 +392,12 @@ fn run_app() -> Result<(), anyhow::Error> {
 
     // Initial setup is done at this point.
 
-    log::info!("Started {} running version {}", &args[0], laurel::VERSION);
+    log::info!(
+        "Started {} running version {} ({})",
+        &args[0],
+        laurel::VERSION,
+        build_id()
+    );
     log::info!(
         "Running with EUID {} using config {}",
         Uid::effective().as_raw(),
@@ -305,7 +418,7 @@ fn run_app() -> Result<(), anyhow::Error> {
         let mut filter_logger =
             Logger::new(&config.filterlog, &dir).context("can't create filterlog logger")?;
         emit_fn_log = move |e: &Event| {
-            if e.filter {
+            if e.is_filtered {
                 filter_logger
                     .log(e)
                     .map_err(|e| anyhow!("Error writing to filter log: {e}"))
@@ -321,7 +434,7 @@ fn run_app() -> Result<(), anyhow::Error> {
     } else {
         log::info!("Dropping filtered audit records");
         emit_fn_drop = move |e: &Event| {
-            if !e.filter {
+            if !e.is_filtered {
                 logger
                     .log(e)
                     .map_err(|e| anyhow!("Error writing to audit log: {e}"))
@@ -337,8 +450,18 @@ fn run_app() -> Result<(), anyhow::Error> {
         None
     };
 
-    coalesce.settings = config.make_coalesce_settings();
-    coalesce.initialize().context("Failed to initialize")?;
+    coalesce = coalesce.with_settings(config.make_coalesce_settings());
+
+    if let Some(state) = statefile_path
+        .as_ref()
+        .and_then(|p| read_state(p, Duration::from_secs(config.state.max_age)))
+    {
+        log::info!("Importing state...");
+        coalesce = coalesce.with_state(state);
+    } else {
+        log::info!("Starting with blank state...");
+        coalesce.initialize().context("Failed to initialize")?;
+    }
 
     let mut line: Vec<u8> = Vec::new();
     let mut stats = Stats::default();
@@ -347,8 +470,8 @@ fn run_app() -> Result<(), anyhow::Error> {
     let statusreport_period = config.statusreport_period.map(Duration::from_secs);
     let mut statusreport_last_t = SystemTime::now();
 
-    let dump_state_period = config.debug.dump_state_period.map(Duration::from_secs);
-    let mut dump_state_last_t = SystemTime::now();
+    let write_state_period = config.state.write_state_period.map(Duration::from_secs);
+    let mut write_state_last_t = SystemTime::now();
 
     sigprocmask(SIG_UNBLOCK, Some(&SigSet::from_iter([SIGHUP])), None)?;
     let hup = Arc::new(AtomicBool::new(false));
@@ -370,7 +493,12 @@ fn run_app() -> Result<(), anyhow::Error> {
                     log::error!("Error {e} processing msg: {line}");
                 }
             }
+
+            if let Some(p) = statefile_path.as_ref() {
+                write_state(p, coalesce.state());
+            }
             coalesce.flush();
+
             log::info!("Restarting...");
             use std::ffi::CString;
             let argv: Vec<CString> = env::args().map(|a| CString::new(a).unwrap()).collect();
@@ -425,7 +553,7 @@ fn run_app() -> Result<(), anyhow::Error> {
             if statusreport_period_t.as_secs() > 0
                 && statusreport_last_t.elapsed()? >= statusreport_period_t
             {
-                log::info!("Laurel version {}", laurel::VERSION);
+                log::info!("Laurel version {} ({})", laurel::VERSION, build_id());
                 log::info!(
                     "Parsing stats (until now): processed {} lines {} events with {} errors in total",
                     &stats.lines, &stats.events, &stats.errors );
@@ -440,16 +568,18 @@ fn run_app() -> Result<(), anyhow::Error> {
             }
         }
 
-        if let (Some(dl), Some(p)) = (&mut debug_logger, &dump_state_period) {
-            if dump_state_last_t.elapsed()? >= *p {
-                coalesce
-                    .dump_state(&mut dl.output)
-                    .map_err(|e| anyhow!("dump state: {e}"))?;
-                dump_state_last_t = SystemTime::now();
+        if let (Some(statefile), Some(p)) = (&config.state.file, &write_state_period) {
+            if write_state_last_t.elapsed()? >= *p {
+                write_state(statefile, coalesce.state());
+                write_state_last_t = SystemTime::now();
             }
         }
     }
 
+    if let Some(p) = statefile_path.as_ref() {
+        write_state(p, coalesce.state());
+    }
+
     // If periodical reports were enabled, stats only contains temporary statistics.
     if let Some(statusreport_period_t) = statusreport_period {
         if statusreport_period_t.as_secs() > 0 {
