@@ -10,12 +10,61 @@ use serde::{
 use crate::coalesce::Settings;
 use crate::label_matcher::LabelMatcher;
 
+fn default_state_file() -> Option<PathBuf> {
+    Some(Path::new("state").into())
+}
+
+fn deserialize_state_file<'de, D>(d: D) -> Result<Option<PathBuf>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    match PathBuf::deserialize(d)? {
+        p if p == PathBuf::default() => Ok(None),
+        p => Ok(Some(p)),
+    }
+}
+
+const fn default_state_max_age() -> u64 {
+    60
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize, Eq, PartialEq)]
+pub struct Statefile {
+    #[serde(
+        default = "default_state_file",
+        deserialize_with = "deserialize_state_file"
+    )]
+    pub file: Option<PathBuf>,
+    #[serde(default)]
+    pub generations: u64,
+    #[serde(rename = "max-age")]
+    #[serde(default = "default_state_max_age")]
+    pub max_age: u64,
+    #[serde(rename = "write-state-period")]
+    pub write_state_period: Option<u64>,
+}
+
+impl Default for Statefile {
+    fn default() -> Self {
+        Self {
+            file: default_state_file(),
+            generations: 0,
+            max_age: default_state_max_age(),
+            write_state_period: None,
+        }
+    }
+}
+
 #[derive(Clone, Default, Debug, Serialize, Deserialize, Eq, PartialEq)]
 pub struct Logfile {
     #[serde(default)]
     pub file: PathBuf,
     #[serde(rename = "read-users")]
     pub users: Option<Vec<String>>,
+    #[serde(rename = "read-groups")]
+    pub groups: Option<Vec<String>>,
+    #[serde(default, rename = "read-other")]
+    pub other: bool,
     pub size: Option<u64>,
     pub generations: Option<u64>,
     #[serde(rename = "line-prefix")]
@@ -24,12 +73,9 @@ pub struct Logfile {
 
 #[derive(Clone, Debug, Default, Serialize, Deserialize, Eq, PartialEq)]
 pub struct Debug {
-    pub log: Option<Logfile>,
     pub inputlog: Option<Logfile>,
     #[serde(rename = "parse-error-log")]
     pub parse_error_log: Option<Logfile>,
-    #[serde(rename = "dump-state-period")]
-    pub dump_state_period: Option<u64>,
 }
 
 #[derive(PartialEq, Eq, Debug, Serialize, Deserialize, Hash)]
@@ -89,12 +135,18 @@ pub struct Enrich {
     pub execve_env: HashSet<String>,
     #[serde(default = "true_value")]
     pub container: bool,
+    #[serde(default)]
+    pub container_info: bool,
+    #[serde(default = "true_value")]
+    pub systemd: bool,
     #[serde(default = "true_value")]
     pub pid: bool,
     #[serde(default = "true_value")]
     pub script: bool,
     #[serde(default = "true_value", rename = "uid-groups")]
     pub uid_groups: bool,
+    #[serde(default)]
+    pub prefix: Option<String>,
 }
 
 impl Default for Enrich {
@@ -102,9 +154,12 @@ impl Default for Enrich {
         Enrich {
             execve_env: execve_env_default(),
             container: true,
+            container_info: false,
+            systemd: true,
             pid: true,
             script: true,
             uid_groups: true,
+            prefix: None,
         }
     }
 }
@@ -209,7 +264,7 @@ impl<'de> Deserialize<'de> for Input {
 
 struct InputVisitor {}
 
-impl<'de> Visitor<'de> for InputVisitor {
+impl Visitor<'_> for InputVisitor {
     type Value = Input;
 
     fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
@@ -242,6 +297,8 @@ pub struct Config {
     #[serde(default)]
     pub marker: Option<String>,
     #[serde(default)]
+    pub state: Statefile,
+    #[serde(default)]
     pub auditlog: Logfile,
     #[serde(default)]
     pub filterlog: Logfile,
@@ -267,19 +324,23 @@ impl Default for Config {
             input: Input::Stdin,
             statusreport_period: None,
             marker: None,
+            state: Statefile {
+                file: Some("state".into()),
+                generations: 3,
+                max_age: 60,
+                write_state_period: None,
+            },
             auditlog: Logfile {
                 file: "audit.log".into(),
-                users: None,
                 size: Some(10 * 1024 * 1024),
                 generations: Some(5),
-                line_prefix: None,
+                ..Logfile::default()
             },
             filterlog: Logfile {
                 file: "filtered.log".into(),
-                users: None,
                 size: Some(10 * 1024 * 1024),
                 generations: Some(5),
-                line_prefix: None,
+                ..Logfile::default()
             },
             debug: Debug::default(),
             transform: Transform::default(),
@@ -328,9 +389,12 @@ impl Config {
                 .map(|s| s.as_bytes().to_vec())
                 .collect(),
             enrich_container: self.enrich.container,
+            enrich_container_info: self.enrich.container_info,
+            enrich_systemd: self.enrich.systemd,
             enrich_pid: self.enrich.pid,
             enrich_script: self.enrich.script,
             enrich_uid_groups: self.enrich.uid_groups,
+            enrich_prefix: self.enrich.prefix.clone(),
             proc_label_keys: self
                 .label_process
                 .label_keys
@@ -388,6 +452,7 @@ statusreport-period = 86400
 [auditlog]
 file = "somefile"
 read-users = ["splunk"]
+[state]
 "#,
         )
         .unwrap();
@@ -403,9 +468,14 @@ read-users = ["splunk"]
             Logfile {
                 file: Path::new("somefile").to_path_buf(),
                 users: Some(vec!["splunk".to_string()]),
-                size: None,
-                generations: None,
-                line_prefix: None,
+                ..Logfile::default()
+            }
+        );
+        assert_eq!(
+            c.state,
+            Statefile {
+                file: Some(Path::new("state").into()),
+                ..Statefile::default()
             }
         );
     }
@@ -421,6 +491,7 @@ read-users = ["splunk"]
             r#"
 [auditlog]
 [filterlog]
+[state]
 [debug]
 [transform]
 [translate]
@@ -435,4 +506,16 @@ read-users = ["splunk"]
         // FIXME This does not work because HashSet ordering is not stable.
         // assert!(toml::to_string(&cfg_default) == toml::to_string(&cfg_empty_sections));
     }
+
+    #[test]
+    fn statefile() {
+        let cfg: Config = toml::de::from_str(
+            r#"
+[state] 
+file = ""
+"#,
+        )
+        .expect("toml parse error");
+        assert_eq!(cfg.state.file, None);
+    }
 }
