@@ -6,7 +6,6 @@ pub mod constants;
 pub mod json;
 pub mod label_matcher;
 pub mod logger;
-pub mod parser;
 pub mod proc;
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 pub mod procfs;
@@ -14,5 +13,7 @@ pub(crate) mod quote;
 pub mod rotate;
 #[cfg(target_os = "linux")]
 pub mod sockaddr;
+#[cfg(test)]
+mod test;
 pub mod types;
 pub mod userdb;
