@@ -19,10 +19,20 @@ pub struct FileRotate {
     pub filesize: u64,
     pub generations: u64,
     pub users: Vec<String>,
+    pub groups: Vec<String>,
+    pub other: bool,
     file: Option<File>,
     offset: u64,
 }
 
+fn ignore_missing(e: Error) -> Result<()> {
+    if e.kind() == std::io::ErrorKind::NotFound {
+        Ok(())
+    } else {
+        Err(e)
+    }
+}
+
 impl FileRotate {
     /// Creates a new [`FileRotate`] instance. This does not involve
     /// any I/O operations; the main file is only created when calling
@@ -33,6 +43,8 @@ impl FileRotate {
             filesize: 0,
             generations: 0,
             users: vec![],
+            groups: vec![],
+            other: false,
             file: None,
             offset: 0,
         }
@@ -50,10 +62,22 @@ impl FileRotate {
         self.users.push(user.into());
         self
     }
+    pub fn with_group(mut self, group: &str) -> Self {
+        self.groups.push(group.into());
+        self
+    }
+    pub fn with_other(mut self, other: bool) -> Self {
+        self.other = other;
+        self
+    }
 
     /// Closes the main file and performs a backup file rotation
     pub fn rotate(&mut self) -> Result<()> {
         log::info!("Rotating {}", self.basename.to_string_lossy());
+        if self.generations == 0 {
+            fs::remove_file(&self.basename).or_else(ignore_missing)?;
+            return Ok(());
+        }
         for suffix in (0..self.generations).rev() {
             let mut old = self.basename.clone();
             match suffix {
@@ -63,7 +87,7 @@ impl FileRotate {
             let mut new = self.basename.clone();
             new.push(format!(".{}", suffix + 1));
             if fs::metadata(&old).is_ok() {
-                fs::rename(old, new)?;
+                fs::rename(old, new).or_else(ignore_missing)?;
             }
         }
         self.file = None;
@@ -80,11 +104,21 @@ impl FileRotate {
             AclEntry::allow_user("", Perm::from_bits_truncate(6), None),
             AclEntry::allow_group("", Perm::from_bits_truncate(4), None),
             #[cfg(any(target_os = "linux", target_os = "freebsd"))]
-            AclEntry::allow_other(Perm::empty(), None),
+            AclEntry::allow_other(
+                if self.other {
+                    Perm::READ
+                } else {
+                    Perm::empty()
+                },
+                None,
+            ),
         ];
         for user in &self.users {
             acl.push(AclEntry::allow_user(user, Perm::READ, None));
         }
+        for group in &self.groups {
+            acl.push(AclEntry::allow_group(group, Perm::READ, None));
+        }
 
         if let Ok(mut f) = OpenOptions::new().append(true).open(&self.basename) {
             setfacl(&[&self.basename], &acl, None).map_err(|e| Error::new(e.kind(), e))?;
@@ -138,3 +172,42 @@ impl Write for FileRotate {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    use nix::unistd::mkdtemp;
+    use std::env::temp_dir;
+    #[test]
+    fn fresh_file() {
+        let td = mkdtemp(&temp_dir().join("laurel-test-XXXXXXXX")).expect("can't create temp dir");
+        let mut fr = FileRotate::new(td.join("logfile"));
+        fr.rotate().expect("rotate");
+        fr.write(b"asdf").expect("write");
+        fr.flush().expect("flush");
+        std::fs::remove_dir_all(td).expect("remove_dir_all");
+    }
+
+    #[test]
+    fn existing() {
+        let td = mkdtemp(&temp_dir().join("laurel-test-XXXXXXXX")).expect("can't create temp dir");
+        std::fs::write(&td.join("logfile"), "asdf").expect("setup");
+        let mut fr = FileRotate::new(&td.join("logfile")).with_generations(3);
+        fr.rotate().expect("rotate");
+        assert!(
+            std::fs::exists(&td.join("logfile.1")).expect("stat"),
+            "after rotate, logfile.1 should exist"
+        );
+        assert!(
+            !std::fs::exists(&td.join("logfile")).expect("stat"),
+            "after rotate, logfile should no longer exist"
+        );
+        fr.write(b"asdf").expect("write");
+        fr.flush().expect("flush");
+        assert!(
+            std::fs::exists(&td.join("logfile")).expect("stat"),
+            "after rotate+write, logfile should exist"
+        );
+        std::fs::remove_dir_all(td).expect("remove_dir_all");
+    }
+}
